@@ -123,17 +123,18 @@ To make the whole thing work, I added the generalLiteral to the
 characterPrimary production.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
+
 import functools
 
 from gavo.utils.parsetricks import (
 	Word, Literal, Optional, alphas, CaselessKeyword,
 	ZeroOrMore, OneOrMore, StringEnd,
-	Forward, nums, 
+	Forward, 
 	CaselessLiteral, ParseException, Regex, sglQuotedString, alphanums,
 	White, 
 	ParseExpression,
@@ -313,7 +314,7 @@ def getADQLGrammarCopy(nodes):
 		g.whitespace = Word(" \t\n")   # need that as a hack now and then to keep
 		                               # numbers and identifiers from sticking
 
-		g.unsignedInteger = Word(nums)
+		g.unsignedInteger = Regex(r"\d+")
 		g.unsignedInteger.setName("unsigned integer")
 		exactNumericRE = r"\d+(\.(\d+)?)?|\.\d+"
 		g.exactNumericLiteral = Regex(exactNumericRE)
@@ -425,6 +426,10 @@ def getADQLGrammarCopy(nodes):
 			| g.arrayReference
 			| g.columnReference 
 			| g.setFunctionSpecification 
+			# this should allow whole character value expressions by the
+			# spec.  If we ever actually want that, we'd have to see how
+			# to avoid infinite recursion.
+			| g.generalLiteral
 			| '(' + g.valueExpression + ')'
 			| g.castSpecification)
 		g.valueExpressionPrimary.setName("value expression")
@@ -506,9 +511,10 @@ def getADQLGrammarCopy(nodes):
 			+ ')')
 
 		g.mocArgs = (
-			Args(g.numericValueExpression) - ',' 
-			- Args(g.geometryValueExpression)
-			| Args(g.stringValueExpression))
+			Args(g.stringValueExpression) 
+			^ Args(
+				g.numericValueExpression) - ',' 
+				- Args(g.geometryValueExpression))
 		g.moc = (CaselessKeyword("MOC")("fName")
 			- '(' + g.mocArgs + ')')
 
@@ -614,8 +620,8 @@ def getADQLGrammarCopy(nodes):
 		g.characterPrimary << (
 			g.stringValueFunction
 			| g.generalLiteral 
-			| g.valueExpressionPrimary 
-			| g.userDefinedFunction)
+			| g.userDefinedFunction
+			| g.valueExpressionPrimary)
 
 		g.timestampFunction = (
 			CaselessKeyword('TIMESTAMP')("fName") - '(' 
@@ -771,8 +777,7 @@ def getADQLGrammarCopy(nodes):
 			+ ZeroOrMore(',' + g.columnReference))("groupby")
 		g.havingClause = (CaselessKeyword("HAVING") 
 			+ g.searchCondition)("having")
-		g.orderingSpecification = (CaselessKeyword( "ASC") 
-			| CaselessKeyword("DESC"))
+		g.orderingSpecification = (Regex("(?i)ASC|DESC"))
 		g.sortKey = g.columnName | g.unsignedInteger
 		g.sortSpecification = g.sortKey + Optional(g.orderingSpecification)
 		g.orderByClause = (CaselessKeyword("ORDER") 
@@ -884,10 +889,9 @@ if __name__=="__main__":  # pragma: no cover
 	import pprint
 	syms, grammar = getADQLGrammar()
 	enableTree(syms)
-	spo = RegularIdentifier(ALL_RESERVED_WORDS)
-	spo.parseString("tablesample")
-	res = syms["regularIdentifier"].parseString(
-		"""tablesample
-"""
+	res = syms["statement"].parseString(
+		"""
+		SELECT MOC(3, point('ICRS', 3, 4)) from spatial
+		"""
 		, parseAll=True)
 	pprint.pprint(res.asList(), stream=sys.stderr)
