@@ -5,7 +5,7 @@ system.
 See the userFunction docstring on how to use these.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -562,54 +562,6 @@ _makeBooleanizer("gavo_vocmatch",
 	"%(1)s IN %(2)s")
 
 
-# TODO: move the basic spectral coversion stuff to base.unitconv
-# (and then fix WIRR's dbadapter.py, too).
-
-def toStdUnit(fromUnit):
-	"""returns a pair of (standard unit, factor) to turn fromUnit into
-	one of J, Hz, or m.
-
-	Unconvertable units raise a UfuncError.
-	"""
-	for stdUnit in ["J", "Hz", "m"]:
-		try:
-			 factor = base.computeConversionFactor(fromUnit, stdUnit)
-		except base.IncompatibleUnits:
-			continue
-		return stdUnit, factor
-	
-	raise common.UfuncError(
-		f"specconv: {fromUnit} is not a spectral unit understood here")
-
-
-SPEC_CONVERSION = {
-	("J", "m"): "h*c/(({expr})*{f})",
-	("J", "Hz"): "({expr})*{f}/h",
-	("J", "J"): "({expr})*{f}",
-	("Hz", "m"): "c/({expr})/{f}",
-	("Hz", "Hz"): "{f}*({expr})",
-	("Hz", "J"): "h*{f}*({expr})",
-	("m", "m"): "{f}*({expr})",
-	("m", "Hz"): "c/({expr})/{f}",
-	("m", "J"): "h*c/({expr})/{f}",}
-
-
-def getSpecExpr(fromUnit, toUnit):
-	"""returns an expression that turns a value in fromUnit to one in toUnit.
-
-	The value in fromUnit is represented by a {} so you can just .format
-	the expression it is in into the resulting string.
-	"""
-	fromStd, fromFactor = toStdUnit(fromUnit)
-	toStd, toFactor = toStdUnit(toUnit)
-	convExpr = SPEC_CONVERSION[fromStd, toStd
-			].replace("h", str(base.PLANCK_H)
-			).replace("c", str(base.LIGHT_C))
-	return "{}/{}".format(
-		convExpr.format(f=fromFactor, expr="{}"),
-		toFactor)
-
-
 class _SpecconvNode(nodes.FieldInfoedNode):
 	"""A node left by the two-argument gavo_specconv.
 
@@ -665,19 +617,22 @@ class _SpecconvNode(nodes.FieldInfoedNode):
 			raise common.UfuncError("specconv: Internal Error: No sentinel found.")
 
 	def addFieldInfo(self, context):
-		nodes.FieldInfoedNode.addFieldInfo(self, context)
+		try:
+			nodes.FieldInfoedNode.addFieldInfo(self, context)
 
-		fromUnit = self._getFromUnit(context)
+			fromUnit = self._getFromUnit(context)
 
-		exprLit = getSpecExpr(fromUnit, self.targetUnit
-			).format("REPLACE_ME")
-		exprNode = nodes.getTreeBuildingGrammar()[0][
-				"userDefinedFunctionParam"
-			].parseString(exprLit, parseAll=True)[0]
-		
-		self._replaceExpr("REPLACE_ME", exprNode)
+			exprLit = base.getSpecExpr(fromUnit, self.targetUnit
+				).format("REPLACE_ME")
+			exprNode = nodes.getTreeBuildingGrammar()[0][
+					"userDefinedFunctionParam"
+				].parseString(exprLit, parseAll=True)[0]
+			
+			self._replaceExpr("REPLACE_ME", exprNode)
 
-		self.fieldInfo = self.fieldInfo.change(unit=self.targetUnit)
+			self.fieldInfo = self.fieldInfo.change(unit=self.targetUnit)
+		except base.IncompatibleUnits as msg:
+			raise common.UfuncError(str(msg))
 	
 	def flatten(self):
 		return self.expr.flatten()
@@ -716,17 +671,20 @@ class _SpecconvNode(nodes.FieldInfoedNode):
 		the translator therefore warns if an existing unit is overridden
 		with a different unit.""")])
 def _gavo_specconv(args):
-	if len(args)==2:
-		newNode = _SpecconvNode(expr=args[0], 
-			targetUnit=getStringLiteral(args[1]))
+	try:
+		if len(args)==2:
+			newNode = _SpecconvNode(expr=args[0], 
+				targetUnit=getStringLiteral(args[1]))
 
-	elif len(args)==3:
-		newNode = _SpecconvNode(expr=args[0], 
-			targetUnit=getStringLiteral(args[2]),
-			forceUnit=getStringLiteral(args[1]))
-	
-	else:
-		raise common.UfuncError("gavo_specconv takes two or three arguments")
+		elif len(args)==3:
+			newNode = _SpecconvNode(expr=args[0], 
+				targetUnit=getStringLiteral(args[2]),
+				forceUnit=getStringLiteral(args[1]))
+		
+		else:
+			raise common.UfuncError("gavo_specconv takes two or three arguments")
+	except base.IncompatibleUnits as msg:
+		raise common.UfuncError(msg)
 
 	raise nodes.ReplaceNode(newNode)
 	
