@@ -2,7 +2,7 @@
 Definition of DC config options and their management including I/O.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -431,9 +431,6 @@ _config = Configuration(
 		ListConfigItem("graphicMimes", 
 			"image/fits,image/jpeg,application/x-votable+xml;content=datalink", 
 			"Media types considered as graphics (for SIAP, mostly)"),
-		StringConfigItem("adsMirror", 
-			"http://ads.g-vo.org",
-			"Root URL of ADS mirror to be used (without a trailing slash)"),
 		IntConfigItem("maxUploadSize",
 			"20000000",
 			"Maximal size of (mainly TAP) file uploads in async requests in bytes;"
