@@ -6,7 +6,7 @@ This is currently very postgres specific.  If we really wanted to
 support some other database, this would need massive refactoring.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -31,8 +31,6 @@ debug = "GAVO_SQL_DEBUG" in os.environ
 import psycopg2
 import psycopg2.extensions
 import psycopg2.pool
-psycopg2.extensions.register_type(psycopg2.extensions.UNICODE)
-psycopg2.extensions.register_type(psycopg2.extensions.UNICODEARRAY)
 
 from psycopg2.extras import DictCursor #noflake: exported name
 
@@ -91,6 +89,12 @@ class SqlArrayAdapter(object):
 	"""
 	def __init__(self, seq):
 		self._seq = seq
+		for item in seq:
+			if item is not None:
+				self.itemType = type(item)
+				break
+		else:
+			self.itemType = None
 
 	def prepare(self, conn):
 		pass
@@ -112,12 +116,22 @@ class SqlArrayAdapter(object):
 		elif isinstance(self._seq, utils.intlist):
 			serializedList[0] = b"NULL::INTEGER"
 
-
 	def getquoted(self):
 		if len(self._seq)==0:
 			return b'NULL'
-		qobjs = [psycopg2.extensions.adapt(o).getquoted() 
-			for o in self._seq]
+
+		if self.itemType and issubclass(self.itemType, str):
+			# I need to be a bit verbose here because psycopg's default
+			# encoding still is latin-1, and it seems there's no better
+			# way to force it to utf-8 than this:
+			qobjs = []
+			for o in self._seq:
+				item = psycopg2.extensions.adapt(o)
+				item.encoding = "utf-8"
+				qobjs.append(item.getquoted())
+		else:
+			qobjs = [psycopg2.extensions.adapt(o).getquoted() 
+				for o in self._seq]
 
 		self._addCastIfNecessary(qobjs)
 
@@ -192,8 +206,8 @@ for numpyType, adapter in [
 	try:
 		psycopg2.extensions.register_adapter(
 			getattr(numpy, numpyType), adapter)
-	except AttributeError:
-		# what's not there we don't need to adapt
+	except AttributeError:  # pragma: no cover
+		# types not present on the python end we don't need to adapt
 		pass
 
 # Override psycopg2's mapping of numeric to decimal, because our
@@ -215,16 +229,7 @@ from psycopg2.extensions import QueryCanceledError #noflake: exported name
 from psycopg2 import Error as DBError
 
 
-def registerAdapter(type, adapter):
-	psycopg2.extensions.register_adapter(type, adapter)
-
-
-def registerType(oid, name, castFunc):
-	newOID = psycopg2.extensions.new_type(oid, name, castFunc)
-	psycopg2.extensions.register_type(newOID)
-
-
-class DebugCursor(psycopg2.extensions.cursor):
+class DebugCursor(psycopg2.extensions.cursor):  # pragma: no cover
 	def execute(self, sql, args=None):
 		print("Executing %s %s"%(id(self.connection), sql))
 		psycopg2.extensions.cursor.execute(self, sql, args)
@@ -389,7 +394,7 @@ class GAVOConnection(psycopg2.extensions.connection):
 			self.execute("RELEASE SAVEPOINT %s"%savepointName)
 
 
-class DebugConnection(GAVOConnection):
+class DebugConnection(GAVOConnection):  # pragma: no cover
 	def cursor(self, *args, **kwargs):
 		kwargs["cursor_factory"] = DebugCursor
 		return psycopg2.extensions.connection.cursor(self, *args, **kwargs)
@@ -421,12 +426,10 @@ def getDBConnection(profile, debug=debug, autocommitted=False):
 	profile will usually be a string naming a profile defined in
 	``GAVO_ROOT/etc``.
 	"""
-	if profile is None:
-		profile = "trustedquery"
 	if isinstance(profile, str):
 		profile = config.getDBProfile(profile)
 
-	if debug:
+	if debug:  # pragma: no cover
 		conn = psycopg2.connect(connection_factory=DebugConnection,
 			**profile.getArgs())
 		print("NEW CONN using %s (%s)"%(profile.name, conn.getPID()), id(conn))
@@ -440,7 +443,7 @@ def getDBConnection(profile, debug=debug, autocommitted=False):
 				**profile.getArgs())
 		except OperationalError as msg:
 			raise utils.ReportableError("Cannot connect to the database server."
-				" The database library reported:\n\n%s"%msg,
+				" The database library reported:\n\n%s"%str(msg),
 				hint="This usually means you must adapt either the access profiles"
 				" in $GAVO_DIR/etc or your database config (in particular,"
 				" pg_hba.conf).")
@@ -641,8 +644,8 @@ class PostgresQueryMixin(object):
 		res = list(self.connection.query(
 			"SELECT reltuples FROM pg_class WHERE oid = %(tableName)s::regclass",
 			locals()))
-		if not res:
-			raise KeyError(tableName)
+		# this is guaranteed to return something because of the ::regclass
+		# cast that will fail for non-existing tables.
 		return int(res[0][0])
 
 	def roleExists(self, role):
@@ -659,13 +662,9 @@ class PostgresQueryMixin(object):
 		tableName may be schema qualified.  If it is not, public is assumed.
 		"""
 		schema, tableName = _parseTableName(tableName, schema)
-		res = list(self.connection.query("SELECT oid FROM pg_class WHERE"
-			" relname=%(tableName)s AND"
-			" relnamespace=(SELECT oid FROM pg_namespace WHERE nspname=%(schema)s)",
-			locals()))
-		if len(res)!=1:
-			raise Error("Table %s does not exist"%tableName)
-		return res[0][0]
+		return list(self.connection.query(
+			"SELECT %(tableName)s::regclass::bigint",
+			{"tableName": f"{schema}.{tableName}"}))[0][0]
 
 	def getTableType(self, tableName, schema=None):
 		"""returns the type of the relation relationName.
@@ -892,6 +891,8 @@ class QuerierMixin(PostgresQueryMixin, StandardQueryMixin):
 		the results) from actual selects.  That's a bad API.  Also note
 		that the timeout is ignored for DDL statements.
 
+		We'll drop this some time in 2023.
+
 		Use either connection.query or connection.execute in new code.
 		"""
 		warnings.warn("You are using querier.query (or perhaps table.query)."
@@ -922,13 +923,6 @@ class QuerierMixin(PostgresQueryMixin, StandardQueryMixin):
 		"""
 		return self.connection.queryToDicts(*args, **kwargs)
 
-	def finish(self):
-		self.connection.commit()
-		self.connection.close()
-
-	def abort(self):
-		self.connection.close()
-
 
 class UnmanagedQuerier(QuerierMixin):
 	"""A simple interface to querying the database through a connection
@@ -1114,7 +1108,7 @@ def _initPsycopg():
 		try:
 			from gavo.utils import pgsphere
 			pgsphere.preparePgSphere(conn)
-		except:
+		except:  # prama: no cover
 			warnings.warn("pgsphere missing -- ADQL, pg-SIAP, and SSA will not work")
 	
 		# Add symmetrised q3c_joins if q3c is in use and the functions are
