@@ -9,11 +9,12 @@ Use the meta.forKeys class decorator to teach meta.py where to
 use your new class.
 """
 
-#c Copyright 2008-2021, the GAVO project
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
+
 import datetime
 import re
 from urllib import parse as urlparse
@@ -287,9 +288,10 @@ class BibcodeMeta(meta.MetaValue):
 	into ADS.
 	"""
 	def _makeADSLink(self, matOb):
-		adsMirror = config.get("web", "adsMirror")
-		return '<a href="%s">%s</a>'%(
-			adsMirror+"/abs/%s"%urlparse.quote(matOb.group(0)),
+		urlBibcode = urlparse.quote(matOb.group(0))
+		adsURL = f"https://ui.adsabs.harvard.edu/abs/{urlBibcode}/abstract"
+		return '<a href="{}">{}</a>'.format(
+			adsURL,
 			stanxml.escapePCDATA(matOb.group(0)))
 
 	def _getContentAsHTML(self, content):
