@@ -13,7 +13,7 @@ Unit tests for this are at
 http://svn.ari.uni-heidelberg.de/svn/gavo/python/trunk/tests/unitconvtest.py
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -666,6 +666,53 @@ def computeColumnConversions(newColumns, oldColumns):
 	return res
 
 
+#################### special handling for spectral unit handling
+
+def toStdSpecUnit(fromUnit):
+	"""returns a pair of (standard unit, factor) to turn fromUnit into
+	one of J, Hz, or m.
+
+	Unconvertable units raise a UfuncError.
+	"""
+	for stdUnit in ["J", "Hz", "m"]:
+		try:
+			 factor = computeConversionFactor(fromUnit, stdUnit)
+		except IncompatibleUnits:
+			continue
+		return stdUnit, factor
+	
+	raise IncompatibleUnits(
+		f"{fromUnit} is not a spectral unit understood here")
+
+
+SPEC_CONVERSION = {
+	("J", "m"): "h*c/(({expr})*{f})",
+	("J", "Hz"): "({expr})*{f}/h",
+	("J", "J"): "({expr})*{f}",
+	("Hz", "m"): "c/({expr})/{f}",
+	("Hz", "Hz"): "{f}*({expr})",
+	("Hz", "J"): "h*{f}*({expr})",
+	("m", "m"): "{f}*({expr})",
+	("m", "Hz"): "c/({expr})/{f}",
+	("m", "J"): "h*c/({expr})/{f}",}
+
+
+def getSpecExpr(fromUnit, toUnit):
+	"""returns an expression that turns a value in fromUnit to one in toUnit.
+
+	The value in fromUnit is represented by a {} so you can just .format
+	the expression it is in into the resulting string.
+	"""
+	fromStd, fromFactor = toStdSpecUnit(fromUnit)
+	toStd, toFactor = toStdSpecUnit(toUnit)
+	convExpr = SPEC_CONVERSION[fromStd, toStd
+			].replace("h", str(PLANCK_H)
+			).replace("c", str(LIGHT_C))
+	return "{}/{}".format(
+		convExpr.format(f=fromFactor, expr="{}"),
+		toFactor)
+
+
 if __name__=="__main__":
 	getUnitGrammar.enableDebuggingOutput()
 	g = getUnitGrammar()
