@@ -5,7 +5,7 @@ twisted templates and rend.Pages.
 See porting guide in the README.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -21,7 +21,7 @@ from twisted.web import resource
 from twisted.web import server
 from twisted.web import template
 
-from .twistedpatch import Raw
+from .twistedpatch import Raw, _ToStan, _NSContext, Tag
 
 NEVOW_NS = 'http://nevow.com/ns/nevow/0.1'
 TWISTED_NS = 'http://twistedmatrix.com/ns/twisted.web.template/0.1'
@@ -42,14 +42,14 @@ class NoDataError(Exception):
     pass
 
 
-class _ToStan(template._ToStan):
+class MyToStan(_ToStan):
     """A SAX parser unifying nevow and twisted.web namespaces.
 
     We also map invisible (nevow) to transparent (t.w).
     """
     def __init__(self, sourceFilename):
-        template._ToStan.__init__(self, sourceFilename)
-        self.prefixMap = template._NSContext()
+        _ToStan.__init__(self, sourceFilename)
+        self.prefixMap = _NSContext()
         self.prefixMap[NEVOW_NS] = "nevow"
         self.prefixMap[TWISTED_NS] = "nevow"
     
@@ -57,13 +57,13 @@ class _ToStan(template._ToStan):
         # overridden to swallow attempts to map prefixes for our
         # standard namespaces
         if uri==NEVOW_NS or uri==TWISTED_NS:
-            self.prefixMap = template._NSContext(self.prefixMap)
+            self.prefixMap = _NSContext(self.prefixMap)
             return
         else:
             if prefix=="nevow":
                 raise Exception("You may not bind the nevow prefix"
                     " to a non-twisted or non-nevow URI")
-            template._ToStan.startPrefixMapping(self, prefix, uri)
+            _ToStan.startPrefixMapping(self, prefix, uri)
 
     def startElementNS(self, namespaceAndName, qName, attrs):
         # regrettably, we also need to map attribute namespaces;
@@ -94,7 +94,7 @@ class _ToStan(template._ToStan):
                 else:
                     fixedAttrs[attName] = val
 
-            el = template.Tag('', 
+            el = Tag('', 
                 render=attrs.get((TWISTED_NS, 'render')),
                 attributes=fixedAttrs,
                 filename=self.sourceFilename,
@@ -107,7 +107,7 @@ class _ToStan(template._ToStan):
         if ns==TWISTED_NS and name=='invisible':
             name = 'transparent'
 
-        return template._ToStan.startElementNS(self,
+        return _ToStan.startElementNS(self,
             (ns, name),
             qName,
             attrs)
@@ -123,7 +123,7 @@ def _flatsaxParse(fl):
     parser.setFeature(handler.feature_external_ges, 0)
     parser.setFeature(handler.feature_external_pes, 0)
 
-    s = _ToStan(getattr(fl, "name", None))
+    s = MyToStan(getattr(fl, "name", None))
     parser.setContentHandler(s)
     parser.setEntityResolver(s)
     parser.setProperty(handler.property_lexical_handler, s)
