@@ -7,12 +7,13 @@ for the first time.
 The features were adding include:
 
 * have nevow tag[] syntax again.
-* to swallow attributes with None values in tags
-* (Ab) use slotData to manage what was nevow:data, pass it into the
+* swallow attributes with None values in tags
+* (Ab)use slotData to manage what was nevow:data, pass it into the
   render method as tag.slotData
+* functions are allowed as render values
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -20,7 +21,23 @@ The features were adding include:
 
 from twisted.web import _flatten
 from twisted.web import _stan 
-from twisted.web import template
+
+try:
+    # this is where we want to monkeypatch twisted.template
+    # in twisted 21+
+    from twisted.web import _template_util as _template_patchtarget
+except ImportError:
+    # this is where to monkeypatch in twisted 20
+    from twisted.web import template as _template_patchtarget
+_ToStan = _template_patchtarget._ToStan
+_NSContext = _template_patchtarget._NSContext
+
+
+def patchTwTemplate(name, obj):
+    """monkeypatches whatever module holds the twisted.web.template
+    definitions.
+    """
+    setattr(_template_patchtarget, name, obj)
 
 
 class _NoData(object):
@@ -36,8 +53,17 @@ _StanTag = _stan.Tag
 class Tag(_StanTag):
     def __init__(self, tagName, attributes=None, children=None, render=None,
                  filename=None, lineNumber=None, columnNumber=None, data=None):
-        _StanTag.__init__(self, tagName, attributes, children, render, 
-            filename, lineNumber, columnNumber)
+        children = children or []
+        attributes = attributes or {}
+        _StanTag.__init__(self, 
+            tagName=tagName, 
+            attributes=attributes, 
+            children=children, 
+            filename=filename or "unknown", 
+            lineNumber=lineNumber or -1, 
+            columnNumber=columnNumber or -1)
+        if render is not None:
+            self.render = render
         self.slotData = data
 
     def __getitem__(self, stuff):
@@ -47,6 +73,9 @@ class Tag(_StanTag):
             return self(stuff)
     
     def __call__(self, *items, **attrs):
+        if "render" in attrs:
+            self.render = attrs.pop("render")
+
         for key, value in list(attrs.items()):
             if value is None:
                 attrs.pop(key)
@@ -83,7 +112,6 @@ class Tag(_StanTag):
         else:
             return self.slotData
 
-
     def _clone(self, obj, deep, newData):
         """
         Clone an arbitrary object; used by L{Tag.clone}.
@@ -104,7 +132,6 @@ class Tag(_StanTag):
         else:
             return obj
 
-
     def clone(self, deep=True, newData=_NoData):
         """
         Return a clone of this tag. If deep is True, clone all of this tag's
@@ -139,9 +166,10 @@ class Tag(_StanTag):
 
         return newtag
 
+
 # we have to monkeypatch a couple of places in twisted.web
 _stan.Tag = Tag
-template.Tag = Tag
+patchTwTemplate("Tag", Tag)
 _flatten.Tag = Tag
 
 
@@ -249,7 +277,7 @@ def _flattenElement(request, root, write, slotData, renderFactory,
         else:
             tagName = root.tagName
         write(tagName)
-        for k, v in _flatten.iteritems(root.attributes):
+        for k, v in root.attributes.items():
             if isinstance(k, str):
                 k = k.encode('ascii')
             write(b' ' + k + b'="')
@@ -319,3 +347,5 @@ def _loadNevowData(request, tag, renderFactory):
     res = tag.clone(True, newData)
     del res.attributes["nevow:data"]
     return res
+
+
