@@ -1024,13 +1024,18 @@ class FileUpload(object):
         # default is ignored here.
 
         if request.fields is None or key not in request.fields:
-            return None, None
+            return self.original.validate(None)
         fileitem = request.fields[key]
+        if not fileitem.file.getvalue():
+            # empty input; this happens in modern cgi for when no
+            # file has been passed in
+            return self.original.validate(None)
+
         name = fileitem.filename
         if isinstance(name, bytes):
             name = name.decode("utf-8", "replace")
         value = (name, fileitem.file)
- 
+
         value = iformal.IFileConvertible(self.original).fromType(value)
         return self.original.validate(value)
 
