@@ -9,7 +9,7 @@ VOTable FIELDs and the corresponding values.
 You should access this module through formats.votable.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -533,9 +533,30 @@ COLUMN_REF_UTYPES = [
 	"stc:astrocoords.velocity3d.value3.c2",
 	"stc:astrocoords.velocity3d.value3.c3",
 	"stc:astrocoords.position2d.epoch",
+	"stc:astrocoords.position3d.epoch",
 ]
 
 
+def _addEpochToCoosys(coosys, utypeMap):
+	"""tries to locate epoch information utypeMap and adds it to the
+	V.COOSYS instance coosys if found.
+	"""
+	for epochBase in [
+			"stc:astrocoords.position2d.", "stc:astrocoords.position3d."]:
+		if epochBase+"epoch" in utypeMap:
+			break
+	else:
+		return
+
+	epVal = utypeMap[epochBase+"epoch"]
+	if not isinstance(epVal, stc.ColRef):
+		# If the epoch is not a column reference, inline it.
+		# (the code below will ignore it in that case)
+		coosys(epoch="%s%s"%(
+			utypeMap.get(epochBase+"epoch.yeardef", "J"),
+			epVal))
+
+
 def _makeCOOSYSFromSTC1(utypeMap, serManager):
 	"""returns a VOTable 1.1 COOSYS element inferred from a map of stc utypes
 	to STC1 values.
@@ -552,14 +573,7 @@ def _makeCOOSYSFromSTC1(utypeMap, serManager):
 	sysId = serManager.makeIdFor(coosys, "system")
 	coosys(ID=sysId)
 
-	if "stc:astrocoords.position2d.epoch" in utypeMap:
-		epVal = utypeMap["stc:astrocoords.position2d.epoch"]
-		if not isinstance(epVal, stc.ColRef):
-			# If the epoch is not a column reference, inline it.
-			# (the code below will ignore it in that case)
-			coosys(epoch="%s%s"%(
-				utypeMap.get("stc:astrocoords.position2d.epoch.yeardef", "J"),
-				epVal))
+	_addEpochToCoosys(coosys, utypeMap)
 	
 	stcFrame = utypeMap.get(
 		"stc:astrocoordsystem.spaceframe.coordrefframe", None)
