@@ -6,7 +6,7 @@ rscdef.builtingrammars.GRAMMAR_REGISTRY (we don't want to import all
 the mess in this package just to make that).
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -66,7 +66,8 @@ class FilteredInputFile(object):
 			"stdout": subprocess.PIPE, 
 			"close_fds": True}
 		if silent:
-			processArgs["stderr"] = open("/dev/null", "w")
+			processArgs["stderr"] = utils.devnull()
+
 		self.process = subprocess.Popen([self.filterCommand], **processArgs)
 		self.fromChild, self.toChild = self.process.stdout, self.process.stdin
 		self.buffer = utils.StreamBuffer(2**14)
@@ -358,8 +359,11 @@ class RowIterator(object):
 				raise
 
 		finally:
-			if self.notify:
-				base.ui.notifySourceFinished()
+			self.finalize()
+
+	def finalize(self):
+		if self.notify:
+			base.ui.notifySourceFinished()
 
 	def _filteredIter(self, baseIter):
 		for row in baseIter:
@@ -486,6 +490,11 @@ class FileRowIterator(RowIterator):
 		self.inputFile = curSrc
 		self.sourceToken = parsingFrom
 
+	def finalize(self):
+		RowIterator.finalize(self)
+		if hasattr(self.inputFile, "close"):
+			self.inputFile.close()
+
 
 class FileRowAttributes(object):
 	"""A mixin for grammars with FileRowIterators.
