@@ -9,7 +9,7 @@ the rest of the DC software.
 Currently, only one kind of DirectGrammar is supported: C boosters.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -102,8 +102,8 @@ class CBooster(object):
 	def _build(self):
 		callArgs = {}
 		if self.silence_for_test:
-			# test instrumentation -- don't worry if the file remains open
-			callArgs["stdout"] = open("/dev/null", "w")
+			# test instrumentation
+			callArgs["stdout"] = utils.devnull()
 		if subprocess.call("make", **callArgs):
 			raise base.ReportableError("Booster build failed, messages above.")
 	
@@ -143,8 +143,7 @@ class CBooster(object):
 
 		pipeArgs = {"shell": True, "stdout": subprocess.PIPE}
 		if self.silence_for_test:
-			# test instrumentation -- don't worry if the file remains open
-			pipeArgs["stderr"] = open("/dev/null", "w")
+			pipeArgs["stderr"] = utils.devnull()
 		self.pipe = subprocess.Popen(shellCommand, **pipeArgs)
 		return self.pipe.stdout
 	
