@@ -6,7 +6,7 @@ testhelpers, too.  Unit test modules should not be forced to import
 this.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -167,13 +167,13 @@ XS = QNamer("http://www.w3.org/2001/XMLSchema")
 
 
 VO_SCHEMATA = [
-		"b2find.xsd",
 		"Characterisation.xsd",
 		"Colstats.xsd",    # remove once it's in VODataService
 		"ConeSearch.xsd",
 		"DaCHS.xsd",
 		"DataModel.xsd",
 		"DocRegExt.xsd",
+		"eudat-core.xsd",
 		"oai_dc.xsd",
 		"OAI-PMH.xsd",
 		"RegistryInterface.xsd",
