@@ -11,7 +11,7 @@ is restricted to https; and with current DaCHS, this means disabling
 http altogether.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -159,6 +159,24 @@ def addUser(conn, username, password, remarks):
 	conn.commit()
 
 
+def changeUser(conn, username, password, remarks=None):
+	"""Changes a user's password and remarks.
+
+	This will raise an error if no such user exists.
+	"""
+	storedForm = hashPassword(password)
+	with conn.cursor() as c:
+		if remarks is None:
+			c.execute("UPDATE dc.users SET password=%(storedForm)s"
+			" WHERE username=%(username)s", locals())
+		else:
+			c.execute("UPDATE dc.users SET password=%(storedForm)s,"
+			" remarks=%(remarks)s WHERE username=%(username)s", locals())
+
+	if not c.rowcount:
+		raise base.ReportableError(f"User {username} does not exist.")
+
+
 def addToGroup(conn, username, groupname):
 	"""Adds a user to a group.
 
