@@ -4,7 +4,7 @@ The datalink core and its numerous helper classes.
 More on this in "Datalink Cores" in the reference documentation.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -76,7 +76,8 @@ class DatalinkDescriptorMixin:
 			semantics = self.forSemantics
 
 		return LinkDef.fromFile(localPath, description, semantics,
-			service=service, contentType=None, suppressMissing=suppressMissing)
+			service=service, contentType=contentType, 
+			suppressMissing=suppressMissing)
 
 	@contextlib.contextmanager
 	def currentSemantics(self, sem):
@@ -264,6 +265,9 @@ class DescriptorGenerator(rscdef.ProcApp):
 	This is what you get when you don't define a descriptor generator
 	in your datalink core.
 
+	Before writing your own, see if one of the predefined descriptor
+	generators work for you; see `Descriptor Generators`_ below.
+
 	The following names are available to the code:
 
 		- pubDID -- the pubDID to be resolved
