@@ -2,7 +2,7 @@
 The schema and XML namespaces for OAI/VOR documents.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -52,8 +52,8 @@ registerPrefix("voe", "http://www.ivoa.net/xml/VOEventRegExt/v2",
 	schemaURL("VOEventRegExt.xsd"))
 registerPrefix("g-colstat", "http://dc.g-vo.org/ColStats-1",
 	schemaURL("Colstats.xsd"))
-registerPrefix("b2f", "http://b2find.eudat.eu/schema/b2f/2.0",
-	schemaURL("b2find.xsd"))
+registerPrefix("eudc", "http://schema.eudat.eu/schema/kernel-1",
+	schemaURL("eudat-core.xsd"))
 registerPrefix("dachs", "http://docs.g-vo.org/schemata/DaCHS.xsd",
 	schemaURL("DaCHS.xsd"))
 
