@@ -2,7 +2,7 @@
 Helpers for resource creation.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -134,7 +134,7 @@ def getParseOptions(validateRows=True, doTableUpdates=False,
 		batchSize=1024, maxRows=None, keepGoing=False, dropIndices=False,
 		dumpRows=False, metaOnly=False, buildDependencies=True,
 		systemImport=False, commitAfterMeta=False, dumpIngestees=False,
-		suppressMeta=False):
+		suppressMeta=False, metaPlusIndex=False):
 	"""returns an object with some attributes set.
 
 	This object is used in the parsing code in dddef.  It's a standin
@@ -166,6 +166,7 @@ def getParseOptions(validateRows=True, doTableUpdates=False,
 	po.commitAfterMeta = commitAfterMeta
 	po.dumpIngestees = dumpIngestees
 	po.suppressMeta = suppressMeta
+	po.metaPlusIndex = metaPlusIndex
 	return po
 
 
