@@ -2,7 +2,7 @@
 Making data out of descriptors and sources.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -303,16 +303,10 @@ class Data(base.MetaMixin, common.ParamMixin):
 					continue
 				t.drop()
 
-	def updateMeta(self, updateIndices=False):
-		"""updates meta information kept in the DB on the contained tables.
-		"""
+	def updateMeta(self):
 		for t in self:
 			if hasattr(t, "updateMeta"):
 				t.updateMeta()
-				if updateIndices:
-					t.dropIndices()
-					t.makeIndices()
-		return self
 
 	def recreateTables(self, connection):
 		"""drops and recreates all table that are onDisk.
