@@ -2,7 +2,7 @@
 Description of columns (and I/O fields).
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -438,6 +438,17 @@ class Values(base.Structure):
 						and statRow[srcKey] is not None):
 					self.feed(destKey, str(statRow[srcKey]), ctx)
 
+	def setOptionsFromDict(self, dist, ctx):
+		"""sets Options from the keys of the dict dist.
+
+		This is primarily for injection of values from the DB into the RD.
+		It will hence not change anything if there are already options
+		defined (which are presumably manually set).
+		"""
+		if self.options:
+			return
+		self.options = makeOptions(*dist.keys())
+
 	def validateOptions(self, value):
 		"""returns false if value isn't either in options or doesn't consist of
 		items in options.
@@ -722,7 +733,8 @@ class Column(ColumnBase):
 	  gavo imp -m will apply changes here; you'll manually have to run
 		``analyze <tablename>`` after that.
 	- statistics -- set this to "no" to keep DaCHS from using this column
-	  in dachs limits.
+	  in dachs limits.  Set this to "enumerate" to make DaCHS collect
+	  the discrete values allowed (currently only supported for strings).
 	- targetType -- for a column containing a URL, the media type of the
 	  resource pointed at.  This is for producing extra annotation for
 	  Aladin and friends as per 
@@ -782,14 +794,19 @@ class Column(ColumnBase):
 		is everything we depend on reasonably complete, and table
 		knows better if we should even bother.
 		"""
-		key = f"colstats:{tableId}:{columnName}"
-		statRow = ctx.getInjected(key, None)
-		if not statRow:
-			return
+		statRow = ctx.getInjected(f"colstats:{tableId}:{columnName}", None)
+		discDist = ctx.getInjected(
+			f"discrete-strings:{tableId}:{columnName}", None)
 
+		if not statRow and not discDist:
+			return
 		if not self.values:
 			self.values = base.makeStruct(Values)
-		self.values.setFromStatRow(statRow, ctx)
+
+		if statRow:
+			self.values.setFromStatRow(statRow, ctx)
+		if discDist:
+			self.values.setOptionsFromDict(discDist, ctx)
 	
 	def getStatistics(self):
 		"""returns a dictionary with COLSTATS (or later VS.column) keys
