@@ -6,7 +6,7 @@ Rowmaker procs are compiled in the namespace defined by this module.
 Maybe we should allow additional modules to be specified in gavorc?
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -48,7 +48,8 @@ from gavo.stc.times import ( #noflake: exported names
 	bYearToDateTime, jdnToDateTime, mjdToDateTime, TTtoTAI, TAItoTT)
 from gavo.utils import codetricks
 from gavo.utils import ( #noflake: exported names
-	dmsToDeg, hmsToDeg, DEG, parseISODT, iterSimpleText, getFileStem,
+	dmsToDeg, hmsToDeg, degToDms, degToHms, makeIAUId,
+	DEG, parseISODT, iterSimpleText, getFileStem,
 	getWCSAxis, getRelativePath, loadPythonModule)
 from gavo.utils import pgsphere #noflake: exported names
 
