@@ -2,7 +2,7 @@
 Output tables and their components.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -34,7 +34,7 @@ class OutputField(rscdef.Column):
 	facility::
 
 	  <outputField name="more" 
-	      select="array[centerAlpha,centerDelta] as more" tablehead="More"
+	      select="array[centerAlpha,centerDelta]" tablehead="More"
 	      description="More exposures near the center of this plate">
 	    <formatter><![CDATA[
 	      return T.a(href=base.makeSitePath("/lswscans/res/positions/q/form?"
