@@ -2,7 +2,7 @@
 Classes and methods to support vizier-type specifications on fields.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -22,16 +22,14 @@ from gavo.utils.parsetricks import (Word, Literal, Optional, Forward,
 	ZeroOrMore, Suppress, ParseException, StringEnd, Regex,
 	OneOrMore, CharsNotIn, pyparsingWhitechars)
 
+
 class ParseNode(object):
 	"""is a parse node, consisting of an operator and children.
 
 	The parse trees returned by the various parse functions are built from
 	these.
 
-	This is an abstract class; concrete derivations need to define
-	a set _standardOperators containing the normal binary operators
-	for their types and a dictionary _sqlEmitters containing functions
-	returning SQL fragments, or override asSQL.
+	This is an abstract class.
 	"""
 	def __init__(self, children, operator):
 		self.children = children
@@ -43,15 +41,15 @@ class ParseNode(object):
 	def __repr__(self):
 		return "(%r %r)"%(self.operator, " ".join([str(c) for c in self.children]))
 
-	def _getSQLKey(self, name, item, sqlPars):
+	def getSQLKey(self, name, item, sqlPars):
 		"""wraps base.getSQLKey and can be overridden in case operand 
 		mangling is necessary.
 
-		Do not call getSQLKey directly from within ParseNodes.
+		Do not call base.getSQLKey directly from within ParseNodes.
 		"""
 		return base.getSQLKey(name, item, sqlPars)
 
-	def _insertChild(self, index, field, sqlPars):
+	def insertChild(self, index, field, sqlPars):
 		"""inserts children[index] into sqlPars with a unique key and returns
 		the key.
 
@@ -61,45 +59,15 @@ class ParseNode(object):
 		if item is None:
 			return None
 		assert not isinstance(item, ParseNode)
-		if field.scaling:
+		if getattr(field, "scaling", None):
 			item *= field.scaling
-		return self._getSQLKey(field.name, item, sqlPars)
+		return self.getSQLKey(field.name, item, sqlPars)
 
-	def asSQL(self, field, sqlPars):
-		if self.operator in self._standardOperators:
-			return "%s %s %%(%s)s"%(field.name, self.operator, 
-				self._insertChild(0, field, sqlPars))
-		else:
-			return self._sqlEmitters[self.operator](self, field, sqlPars)
 
 
 class NumericNode(ParseNode):
 	"""A node containing numeric operands (floats or dates).
 	"""
-	def _emitBinop(self, field, sqlPars):
-		return base.joinOperatorExpr(self.operator,
-			[c.asSQL(field, sqlPars) for c in self.children])
-		
-	def _emitUnop(self, field, sqlPars):
-		operand = self.children[0].asSQL(field, sqlPars)
-		if operand:
-			return "%s (%s)"%(self.operator, operand)
-
-	def _emitEnum(self, field, sqlPars):
-		return "%s IN (%s)"%(field.name, ", ".join([
-					"%%(%s)s"%self._insertChild(i, field, sqlPars) 
-				for i in range(len(self.children))]))
-
-	_standardOperators = set(["=", ">=", ">", "<=", "<"])
-	_sqlEmitters = {
-		'..': lambda self, field, sqlPars: "%s BETWEEN %%(%s)s AND %%(%s)s"%(
-			field.name, self._insertChild(0, field, sqlPars), 
-			self._insertChild(1, field, sqlPars)),
-		'AND': _emitBinop,
-		'OR': _emitBinop,
-		'NOT': _emitUnop,
-		',': _emitEnum,
-	}
 
 
 class DateNode(ParseNode):
@@ -109,77 +77,10 @@ class DateNode(ParseNode):
 	1000 .. 3000 is a julian year, 10000 ... 100000 is an MJD and
 	2000000 .. 4000000 is a JD.
 	"""
-	def _expandDate(self, arg):
-		# returns the last second of arg if it looks like a day (i.e., hms=0)
-		# this is to fix postgres' behaviour when comparing timestamps and
-		# dates (e.g., 1990-01-01 < 1990-01-01T00:00:01).
-		if arg.hour==arg.minute==arg.second==0:
-			return arg.replace(hour=23, 
-				minute=59, second=59)
-		return arg
-
-	def _emitBinop(self, field, sqlPars):
-		return base.joinOperatorExpr(self.operator,
-			[c.asSQL(field, sqlPars) for c in self.children])
-
-	def _emitUnop(self, field, sqlPars):
-		operand = self.children[0].asSQL(field, sqlPars)
-		if operand:
-			return "%s (%s)"%(self.operator, operand)
-
-	def _emitRange(self, field, sqlPars):
-		return "%s BETWEEN %%(%s)s AND %%(%s)s"%(
-			field.name, 
-			self._insertChild(0, field, sqlPars), 
-			self._insertChild(1, field, sqlPars))
-
-	def _emitEnum(self, field, sqlPars):
-		return base.joinOperatorExpr("OR", 
-			(self.__class__([child], "=").asSQL(field, sqlPars) 
-				for child in self.children))
-
-	def _emitSimple(self, field, sqlPars):
-		# return a simple comparison with the date itself
-		return "%s %s %%(%s)s"%(field.name, self.operator, 
-			self._insertChild(0, field, sqlPars))
-
-	def _emitEqual(self, field, sqlPars):
-		if self.children[0].hour==self.children[0].minute==0:
-			return "%s BETWEEN %%(%s)s AND %%(%s)s"%(
-				field.name, 
-				self._getSQLKey(field.name, self.children[0], sqlPars),
-				self._getSQLKey(field.name, 
-					self._expandDate(self.children[0]), sqlPars))
-
-		else:
-			return self._emitSimple(field, sqlPars)
-
-	def _emitExpanded(self, field, sqlPars):
-		# return a simple comparison with date's midnight
-		self.children[0] = self._expandDate(self.children[0])
-		return "%s %s %%(%s)s"%(field.name, self.operator, 
-			self._insertChild(0, field, sqlPars))
-
-	def asSQL(self, field, sqlPars):
-		return self._sqlEmitters[self.operator](self, field, sqlPars)
-
-	_sqlEmitters = {
-		'..': _emitRange,
-		'AND': _emitBinop,
-		'OR': _emitBinop,
-		'NOT': _emitUnop,
-		',': _emitEnum,
-		"=": _emitEqual,
-		">=": _emitSimple,
-		"<=": _emitExpanded,
-		"<": _emitSimple,
-		">": _emitExpanded,
-		"<=": _emitExpanded,
-	}
 
 
 class MJDNode(DateNode):
-	def _getSQLKey(self, name, item, sqlPars):
+	def getSQLKey(self, name, item, sqlPars):
 		if (isinstance(item, datetime.datetime) 
 				or isinstance(item, datetime.date)):
 			item = stc.dateTimeToMJD(item)
@@ -189,85 +90,6 @@ class MJDNode(DateNode):
 class StringNode(ParseNode):
 	"""A node containing string operands.
 	"""
-	def asSQL(self, field, sqlPars):
-		if self.operator=="[":
-			return "[%s]"%self.children[0]
-		if self.operator in self._nullOperators:
-			return self._nullOperators[self.operator]
-		else:
-			return super(StringNode, self).asSQL(field, sqlPars)
-
-	_metaEscapes = {
-		"|": r"\|",
-		"*": r"\*",
-		"+": r"\+",
-		"(": r"\(",
-		")": r"\)",
-		"[": r"\[",
-		"%": r"\%",
-		"_": r"\_",
-		"\\\\": "\\\\",
-	}
-	_escapeRE = re.compile("[%s]"%"".join(list(_metaEscapes.keys())))
-	# The backslash in _metaEscapes is escaped to make _escapeRE work,
-	# but of course I need to replace the unescaped version.
-	_metaEscapes.update({"\\": "\\\\"})
-
-	def _escapeSpecials(self, aString):
-		"""returns aString with SQL RE metacharacters escaped.
-		"""
-		return self._escapeRE.sub(lambda mat: self._metaEscapes[mat.group()],
-			aString)
-
-	def _makePattern(self, field, sqlPars):
-		parts = []
-		for child in self.children:
-			if isinstance(child, str):
-				parts.append(self._escapeSpecials(child))
-			else:
-				parts.append(child.asSQL(field, sqlPars))
-		return "^%s$"%("".join(parts))
-
-	_patOps = {
-		"~": "~*",
-		"=": "~",
-		"!~": "!~*",
-		"!": "!~",
-		"=~": "~*",
-	}
-	def _emitPatOp(self, field, sqlPars):
-		pattern = self._makePattern(field, sqlPars)
-		return "%s %s %%(%s)s"%(field.name, self._patOps[self.operator],
-			self._getSQLKey(field.name, pattern, sqlPars))
-
-	def _emitEnum(self, field, sqlPars):
-		query = "%s IN (%s)"%(field.name, ", ".join([
-					"%%(%s)s"%self._insertChild(i, field, sqlPars) 
-				for i in range(len(self.children))]))
-		if self.operator=="!=,":
-			query = "NOT (%s)"%query
-		return query
-
-	_translatedOps = {
-		"==": "=",
-	}
-	def _emitTranslatedOp(self, field, sqlPars):
-		return "%s = %%(%s)s"%(field.name, 
-			self._insertChild(0, field, sqlPars))
-
-	_nullOperators = {"*": ".*", "?": "."}
-	_standardOperators = set(["<", ">", "<=", ">=", "!="])
-	_sqlEmitters = {
-		"~": _emitPatOp,
-		"=": _emitPatOp,
-		"!~": _emitPatOp,
-		"!": _emitPatOp,
-		"=~": _emitPatOp,  # this happens to work because of pattern escaping
-		"=,": _emitEnum,
-		"=|": _emitEnum,
-		"!=,": _emitEnum,
-		"==": _emitTranslatedOp,
-		}
 
 
 def _getNodeFactory(op, nodeClass):
@@ -287,17 +109,17 @@ def _makeNotNodeFactory(nodeClass):
 	return _makeNotNode
 
 
-def _makePmNode(s, loc, toks):
+def makePmNode(s, loc, toks):
 	return NumericNode([toks[0]-toks[1], toks[0]+toks[1]], "..")
 
 
-def _makeDatePmNode(s, loc, toks):
+def makeDatePmNode(s, loc, toks):
 	"""returns a +/- node for dates, i.e., toks[1] is a float in days.
 	"""
 	days = datetime.timedelta(days=toks[1])
 	return DateNode([toks[0]-days, toks[0]+days], "..")
 
-def _makeMJDPmNode(s, loc, toks):
+def makeMJDPmNode(s, loc, toks):
 	"""returns a +/- node for MJDs, i.e., toks[1] is a float in days, and 
 	an MJDNode must be returned.
 	"""
@@ -305,7 +127,6 @@ def _makeMJDPmNode(s, loc, toks):
 	return MJDNode([toks[0]-days, toks[0]+days], "..")
 
 
-
 def _getBinopFactory(op, nodeClass):
 	def _(s, loc, toks):
 		if len(toks)==1:
@@ -420,14 +241,14 @@ _DATE_LITERAL_DT = Regex(_DATE_REGEX).addParseAction(parseDateTime)
 
 
 def parseNumericExpr(str, baseSymbol=getComplexGrammar(floatLiteral, 
-		_makePmNode)):
+		makePmNode)):
 	"""returns a parse tree for vizier-like expressions over floats.
 	"""
 	return utils.pyparseString(baseSymbol, str)[0]
 
 
 def parseDateExpr(str, baseSymbol=getComplexGrammar(_DATE_LITERAL_DT,
-		_makeDatePmNode, floatLiteral, DateNode)):
+		makeDatePmNode, floatLiteral, DateNode)):
 	"""returns a parse tree for vizier-like expressions over ISO dates.
 
 	Note that the semantic validity of the date (like, month<13) is not
@@ -437,7 +258,7 @@ def parseDateExpr(str, baseSymbol=getComplexGrammar(_DATE_LITERAL_DT,
 
 
 def parseDateExprToMJD(str, baseSymbol=getComplexGrammar(_DATE_LITERAL_DT,
-		_makeMJDPmNode, floatLiteral, MJDNode)):
+		makeMJDPmNode, floatLiteral, MJDNode)):
 	"""returns a parse tree for vizier-like expression of ISO dates with
 	parsed values in MJD.
 	"""
@@ -518,10 +339,303 @@ def parseStringExpr(str, baseSymbol=getStringGrammar()):
 	return utils.pyparseString(baseSymbol, str)[0]
 
 
-def _makeFactory(parser):
+class _Flattener:
+	"""An abstract base class for things turning our parse trees into SQL.
+
+	Concrete derivations need to define a set _standardOperators containing the
+	normal binary operators for their types and a dictionary _sqlEmitters
+	containing functions returning SQL fragments; of course, they
+	can also completey override getSQLFor.
+	"""
+	def getSQLFor(self, node, field, sqlPars):
+		"""returns SQL for a parse node node over an InputKey field.
+
+		sqlPars is the dict that is later passed to conn.execute and
+		will be modified here.
+		"""
+		if node.operator in self._standardOperators:
+			return "{} {} %({})s".format(
+				field.name, 
+				node.operator, 
+				node.insertChild(0, field, sqlPars))
+		else:
+			return self._sqlEmitters[node.operator](
+				self, node, field, sqlPars)
+
+
+class NumericFlattener(_Flattener):
+	"""A _Flattener for float- and int-valued columns.
+	"""
+	def _emitBinop(self, node, field, sqlPars):
+		return base.joinOperatorExpr(node.operator,
+			[self.getSQLFor(c, field, sqlPars) for c in node.children])
+		
+	def _emitUnop(self, node, field, sqlPars):
+		operand = self.getSQLFor(node.children[0], field, sqlPars)
+		if operand:
+			return "{} ({})".format(node.operator, operand)
+
+	def _emitEnum(self, node, field, sqlPars):
+		return "{} IN ({})".format(
+			field.name, 
+			", ".join("%({})s".format(node.insertChild(i, field, sqlPars))
+				for i in range(len(node.children))))
+
+	_standardOperators = set(["=", ">=", ">", "<=", "<"])
+	_sqlEmitters = {
+		'..': lambda self, node, field, sqlPars: 
+			"{} BETWEEN %({})s AND %({})s".format(
+				field.name, 
+				node.insertChild(0, field, sqlPars), 
+				node.insertChild(1, field, sqlPars)),
+		'AND': _emitBinop,
+		'OR': _emitBinop,
+		'NOT': _emitUnop,
+		',': _emitEnum,
+	}
+
+
+class NumericIntervalFlattener(NumericFlattener):
+	"""A _Flattener for matching against min/max pairs of columns.
+
+	Here, field is this (min_column, max_column) pair.
+
+	This currently isn't used anywhere in DaCHS itself.  You can use
+	it in custom phraseMakers, though; cf. :samplerd:`obsform/q`
+	"""
+	def _emitEnum(self, node, field, sqlPars):
+		return base.joinOperatorExpr("OR",
+			[self._emitEquality(node, field, sqlPars, useChild=childInd)
+				for childInd, _ in enumerate(node.children)])
+
+	def _emitInterval(self, node, field, sqlPars):
+		# interval condition: overlaps
+		return ("{h1}>=%({l2})s AND %({h2})s>={l1}"
+			" AND {l1}<={h1} AND %({l2})s<=%({h2})s").format(
+			l1=field[0].name,
+			h1=field[1].name,
+			l2=node.insertChild(0, field[0], sqlPars),
+			h2=node.insertChild(1, field[1], sqlPars))
+
+	def _emitEquality(self, node, field, sqlPars, useChild=0):
+		# equality is interpreted as "contains" here
+		return "%({})s BETWEEN {} AND {}".format(
+			node.insertChild(useChild, field[0], sqlPars),
+			field[0].name,
+			field[1].name)
+	
+	def _emitLarger(self, node, field, sqlPars):
+		# larger/larger equal is, perhaps a bit funkily, larger than
+		# upper/lower bound.
+		if "=" in node.operator:
+			limitField = field[1]
+		else:
+			limitField = field[0]
+
+		return "{} > %({})s".format(
+			limitField.name,
+			node.insertChild(0, limitField, sqlPars))
+
+	def _emitSmaller(self, node, field, sqlPars):
+		# smaller/smaller equal is, perhaps a bit funkily, smaller than
+		# lower/upper bound.
+		if "=" in node.operator:
+			limitField = field[0]
+		else:
+			limitField = field[1]
+
+		return "{} < %({})s".format(
+			limitField.name,
+			node.insertChild(0, limitField, sqlPars))
+
+	_standardOperators = set()
+	_sqlEmitters = {
+		'..': _emitInterval,
+		'=': _emitEquality,
+		'>=': _emitLarger,
+		'>': _emitLarger,
+		'<': _emitSmaller,
+		'<=': _emitSmaller,
+		'AND': NumericFlattener._emitBinop,
+		'OR': NumericFlattener._emitBinop,
+		'NOT': NumericFlattener._emitUnop,
+		',': _emitEnum,
+	}
+
+
+class DateFlattener(_Flattener):
+	"""A _Flattener producing SQL for data-like columns.
+
+	This is supposed to work for timestamp-valued and MJD-valued columns
+	alike.  The difference is handled on the level of nodes (DateNode
+	vs. MJDNode).
+	"""
+	def _expandDate(self, arg):
+		# returns the last second of arg if it looks like a day (i.e., hms=0)
+		# this is to fix postgres' behaviour when comparing timestamps and
+		# dates (e.g., 1990-01-01 < 1990-01-01T00:00:01).
+		if arg.hour==arg.minute==arg.second==0:
+			return arg.replace(hour=23, 
+				minute=59, second=59)
+		return arg
+
+	def _emitBinop(self, node, field, sqlPars):
+		return base.joinOperatorExpr(node.operator,
+			[self.getSQLFor(c, field, sqlPars) for c in node.children])
+
+	def _emitUnop(self, node, field, sqlPars):
+		operand = self.getSQLFor(node.children[0], field, sqlPars)
+		if operand:
+			return "%s (%s)"%(node.operator, operand)
+
+	def _emitRange(self, node, field, sqlPars):
+		return "{} BETWEEN %({})s AND %({})s".format(
+			field.name, 
+			node.insertChild(0, field, sqlPars), 
+			node.insertChild(1, field, sqlPars))
+
+	def _emitEnum(self, node, field, sqlPars):
+		return base.joinOperatorExpr("OR", 
+			[self.getSQLFor(node.__class__([child], "="), field, sqlPars)
+				for child in node.children])
+
+	def _emitSimple(self, node, field, sqlPars):
+		# return a simple comparison with the date itself
+		return "{} {} %({})s".format(field.name, node.operator, 
+			node.insertChild(0, field, sqlPars))
+
+	def _emitEqual(self, node, field, sqlPars):
+		if node.children[0].hour==node.children[0].minute==0:
+			# expand this to a condition covering the whole day
+			return "{} BETWEEN %({})s AND %({})s".format(
+				field.name, 
+				node.getSQLKey(field.name, node.children[0], sqlPars),
+				node.getSQLKey(field.name, 
+					self._expandDate(node.children[0]), sqlPars))
+
+		else:
+			return self._emitSimple(node, field, sqlPars)
+
+	def _emitExpanded(self, node, field, sqlPars):
+		# return a simple comparison with date's midnight
+		node.children[0] = self._expandDate(node.children[0])
+		return "{} {} %({})s".format(
+			field.name, 
+			node.operator, 
+			node.insertChild(0, field, sqlPars))
+
+	_standardOperators = set()
+	_sqlEmitters = {
+		'..': _emitRange,
+		'AND': _emitBinop,
+		'OR': _emitBinop,
+		'NOT': _emitUnop,
+		',': _emitEnum,
+		"=": _emitEqual,
+		">=": _emitSimple,
+		"<=": _emitExpanded,
+		"<": _emitSimple,
+		">": _emitExpanded,
+		"<=": _emitExpanded,
+	}
+
+
+class StringFlattener(_Flattener):
+	"""A _Flattener turning String-like Vizier expressions into
+	SQL conditions against string-valued columns.
+	"""
+	_metaEscapes = {
+		"|": r"\|",
+		"*": r"\*",
+		"+": r"\+",
+		"(": r"\(",
+		")": r"\)",
+		"[": r"\[",
+		"%": r"\%",
+		"_": r"\_",
+		"\\\\": "\\\\",
+	}
+	_escapeRE = re.compile("[%s]"%"".join(list(_metaEscapes.keys())))
+	# The backslash in _metaEscapes is escaped to make _escapeRE work,
+	# but of course I need to replace the unescaped version.
+	_metaEscapes.update({"\\": "\\\\"})
+
+	def _escapeSpecials(self, aString):
+		"""returns aString with SQL RE metacharacters escaped.
+		"""
+		return self._escapeRE.sub(
+			lambda mat: self._metaEscapes[mat.group()],
+			aString)
+
+	def _makePattern(self, node, field, sqlPars):
+		parts = []
+		for child in node.children:
+			if isinstance(child, str):
+				parts.append(self._escapeSpecials(child))
+			else:
+				parts.append(self.getSQLFor(child, field, sqlPars))
+		return "^%s$"%("".join(parts))
+
+	_patOps = {
+		"~": "~*",
+		"=": "~",
+		"!~": "!~*",
+		"!": "!~",
+		"=~": "~*",
+	}
+	def _emitPatOp(self, node, field, sqlPars):
+		pattern = self._makePattern(node, field, sqlPars)
+		return "{} {} %({})s".format(
+			field.name, 
+			self._patOps[node.operator],
+			node.getSQLKey(field.name, pattern, sqlPars))
+
+	def _emitEnum(self, node, field, sqlPars):
+		query = "{} IN ({})".format(
+			field.name, 
+			", ".join(
+					"%({})s".format(node.insertChild(i, field, sqlPars))
+				for i in range(len(node.children))))
+
+		if node.operator=="!=,":
+			query = "NOT ({})".format(query)
+		return query
+
+	_translatedOps = {
+		"==": "=",
+	}
+	def _emitTranslatedOp(self, node, field, sqlPars):
+		return "{} = %({})s".format(field.name, 
+			node.insertChild(0, field, sqlPars))
+
+	_nullOperators = {"*": ".*", "?": "."}
+	_standardOperators = set(["<", ">", "<=", ">=", "!="])
+	_sqlEmitters = {
+		"~": _emitPatOp,
+		"=": _emitPatOp,
+		"!~": _emitPatOp,
+		"!": _emitPatOp,
+		"=~": _emitPatOp,  # this happens to work because of pattern escaping
+		"=,": _emitEnum,
+		"=|": _emitEnum,
+		"!=,": _emitEnum,
+		"==": _emitTranslatedOp,
+		}
+	
+	def getSQLFor(self, node, field, sqlPars):
+		if node.operator=="[":
+			return "[%s]"%node.children[0]
+		if node.operator in self._nullOperators:
+			return self._nullOperators[node.operator]
+		else:
+			return _Flattener.getSQLFor(self, node, field, sqlPars)
+
+
+def _makeFactory(parser, flattener):
 	def factory(field, val, sqlPars):
 		try:
-			return parser(val).asSQL(field, sqlPars)
+			tree = parser(val)
+			return flattener.getSQLFor(tree, field, sqlPars)
 		except ParseException:
 			raise base.ui.logOldExc(utils.ValidationError(
 				"Invalid input for type %s (see help for valid type literals)"%
@@ -529,14 +643,16 @@ def _makeFactory(parser):
 	return factory
 
 
+_dateFlattener = DateFlattener()
+
 sqlmunge.registerSQLFactory("vexpr-float",
-	_makeFactory(parseNumericExpr))
+	_makeFactory(parseNumericExpr, NumericFlattener()))
 sqlmunge.registerSQLFactory("vexpr-date",
-	_makeFactory(parseDateExpr))
+	_makeFactory(parseDateExpr, _dateFlattener))
 sqlmunge.registerSQLFactory("vexpr-mjd",
-	_makeFactory(parseDateExprToMJD))
+	_makeFactory(parseDateExprToMJD, _dateFlattener))
 sqlmunge.registerSQLFactory("vexpr-string",
-	_makeFactory(parseStringExpr))
+	_makeFactory(parseStringExpr, StringFlattener()))
 
 
 class ToVexprConverter(typesystems.FromSQLConverter):
