@@ -2,7 +2,7 @@
 Common error handling facilities for user interface components.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -163,7 +163,7 @@ def raiseAndCatch(opts=None, output=outputError):
 			messages.append("While executing DB query:\n")
 			messages.append("  "+utils.debytify(msg.cursor.query))
 		messages.append("\nThe database engine reported:\n")
-		messages.append("  "+utils.debytify(msg.pgerror))
+		messages.append("  "+utils.debytify(msg.pgerror or b"(Nothing)"))
 
 	else:
 		if hasattr(msg, "excRow"):
