@@ -13,7 +13,7 @@ upgrade does everything required to bring the what's in the database in sync
 with the code (or so I hope).
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -37,7 +37,7 @@ class _COMMIT(object):
 	"""
 
 
-CURRENT_SCHEMAVERSION = 29
+CURRENT_SCHEMAVERSION = 30
 
 
 def getColumnNamesFor(tableId, connection):
@@ -857,6 +857,28 @@ class To29Upgrade(Upgrader):
 					f" {rdId}: {msg}.  Please fix manually.")
 
 
+class To30Upgrader(Upgrader):
+	version = 29
+
+	@classmethod
+	def u_10_make_stats_index(cls, connection):
+		"""Updating indices on dc.simple_col_stats"""
+		table = rsc.TableForDef(
+			base.resolveCrossId("//dc_tables#simple_col_stats"),
+			connection=connection)
+		table.dropIndices()
+		table.makeIndices()
+
+	@classmethod
+	def u_20_create_discrete_text_values(cls, connection):
+		"""Creating dc.discrete_string_values"""
+		# do this 
+		rsc.TableForDef(
+			base.resolveCrossId("//dc_tables#discrete_string_values"),
+			connection=connection,
+			create=True)
+
+
 def iterStatements(startVersion, endVersion=CURRENT_SCHEMAVERSION, 
 		upgraders=None):
 	"""yields all upgraders from startVersion to endVersion in sequence.
