@@ -3,13 +3,14 @@ Functions dealing with compilation and introspection of python and
 external code.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
 
 
 import ast
+import atexit
 import contextlib
 import importlib.util
 import io
@@ -412,7 +413,7 @@ def silence(errToo=False):
 	"""
 	with _SILENCE_LOCK:
 		realstdout = sys.stdout
-		sys.stdout = open("/dev/null", "w")
+		sys.stdout = devnull()
 		if errToo:
 			realstderr = sys.stderr
 			sys.stderr = sys.stdout
@@ -621,7 +622,7 @@ def importModule(modName):
 	The function returns the imported module.
 	"""
 	# ward against exploits (we're about to use exec): check syntax
-	if not re.match("([A-Za-z_]+)(\.[A-Za-z_]+)*", modName):
+	if not re.match(r"([A-Za-z_]+)(\.[A-Za-z_]+)*", modName):
 		raise excs.Error("Invalid name in internal import: %s"%modName)
 	parts = modName.split(".")
 	vars = {}
@@ -757,6 +758,21 @@ def addDefaults(dataDict, defaultDict):
 			dataDict[key] = value
 
 
+@functools.lru_cache(None)
+def devnull():
+	"""returns a (string-) writable /dev/null.
+
+	This always returns the same object, and to placate resource warnings,
+	the file will be closed before exiting the programme; the close method
+	of the returned thing is a no-op.
+	"""
+	devnull = open("/dev/null", "w")
+	closeIt = devnull.close
+	devnull.close = lambda: None
+	atexit.register(closeIt)
+	return devnull
+
+
 def memoizeOn(onObject, generatingObject, generatingFunction, *args):
 	"""memoizes the result of generatingFunction on onObject.
 
