@@ -7,7 +7,7 @@ and you're done.
 All native representation is in rad.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -269,7 +269,7 @@ class SCircle(PgSAdapter):
 		pixels = healpy.query_disc(vec=self.center.asUnitSphereVec(),
 			radius=self.radius,
 			nside=2**order, nest=True, inclusive=inclusive)
-		return SMoc.fromCells(order, pixels)
+		return SMoc.fromCells(order, pixels, maxOrder=order)
 
 
 class SPoly(PgSAdapter):
@@ -343,15 +343,12 @@ class SPoly(PgSAdapter):
 		"""returns an SMoc instance for this polygon.
 
 		order is the maximum order of the moc returned.
-
-		This applies a method developed by FX Pineau, as the healpy-provided
-		query_polygon is to restrictive in practice.
 		"""
 		import healpy
 		vertices = [p.asUnitSphereVec() for p in self.points]
 		pixels = healpy.query_polygon(vertices=vertices,
 			nside=2**order, nest=True, inclusive=inclusive)
-		return SMoc.fromCells(order, pixels)
+		return SMoc.fromCells(order, pixels, maxOrder=order)
 
 	def getCenter(self):
 		"""returns an estimate for some sort of center of this polygon.
@@ -483,10 +480,19 @@ class SMoc(PgSAdapter):
 
 	def __init__(self, moc):
 		self.moc = moc
+		if not hasattr(self.moc, "explicitMaxOrder"):
+			self.moc.explicitMaxOrder = None
 
 	_orderHeadRE = re.compile(r"(\d+)/")
 	_rangeRE = re.compile(r"\s*(\d+)(?:-(\d+))?\s*$")
 
+	@property
+	def maxOrder(self):
+		if self.moc.explicitMaxOrder is None:
+			return self.moc.order
+		else:
+			return self.moc.explicitMaxOrder
+
 	@classmethod
 	def _parseCells(cls, cellLiteral, firstPos):
 		"""returns a sequence of cells from a MOC cell literal.
@@ -532,22 +538,30 @@ class SMoc(PgSAdapter):
 				texttricks.makeEllipsis(literal, 40))
 
 		moc = pymoc.MOC()
+		moc.explicitMaxOrder = None
 
 		for openMat, closeMat in codetricks.iterRanges(seps):
 			order = int(openMat.group(1))
-			cells = cls._parseCells(literal[openMat.end():closeMat.start()],
+			cells = cls._parseCells(
+				literal[openMat.end():closeMat.start()],
 				openMat.end())
-			moc.add(order, cells)
+			if cells:
+				moc.add(order, cells)
+			else:
+				moc.explicitMaxOrder = order
 
 		return cls(moc)
 	
 	@classmethod
-	def fromCells(cls, order, pixels):
+	def fromCells(cls, order, pixels, maxOrder=None):
 		"""returns a SMoc instance from a collection of pixels at order.
+
+		Pass maxOrder to set an explicit max order for the resulting MOC.
 		"""
 		import pymoc
 		
 		moc = pymoc.MOC(order=order, cells=pixels)
+		moc.explicitMaxOrder = maxOrder
 		moc.normalize()
 		return cls(moc)
 
@@ -581,6 +595,9 @@ class SMoc(PgSAdapter):
 
 			ranges.append(self._formatASCIIRange(rmin, rmax))
 			parts.append("%d/%s"%(order, " ".join(ranges)))
+		
+		if self.moc.explicitMaxOrder is not None:
+			parts.append(f"{self.moc.explicitMaxOrder}/")
 
 		return " ".join(parts)
 
@@ -618,6 +635,10 @@ class SMoc(PgSAdapter):
 		"""
 		moc = self.moc.copy()
 		moc.normalize(order)
+
+		if self.moc.explicitMaxOrder is not None:
+			moc.explicitMaxOrder = min(self.moc.explicitMaxOrder, order)
+
 		return self.__class__(moc)
 
 	def getPlot(self, **kwargs):
