@@ -2,7 +2,7 @@
 A renderer to do RD-based maintainance.
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -18,7 +18,6 @@ from gavo import base
 from gavo import formal
 from gavo import stc
 from gavo import svcs
-from gavo.protocols import creds
 from gavo.web import common
 from gavo.web import grend
 
@@ -172,87 +171,3 @@ class AdminRenderer(formal.ResourceWithForm,
 			T.body[
 				T.p["Admin services are only available with a admin.html template"]]
 		])
-
-
-class AuthcheckRenderer(grend.ServiceBasedPage):
-	"""A renderer that lets people can use to see if a username/password
-	name is valid on this service (and if there's auth on at all).
-
-	It returns text/plain, with 403 with a "NO" payload if the pair doesn't 
-	work, and 200 with a "PROBABLY YES" payload if it does.
-
-	On an unrestricted service, this check is if password matches username.
-	That's mainly for services like our UWS that let people put in
-	gratuitous auth.
-
-	This is (at least for now) highly experimental and will presumably go away
-	again.  If you find you need it, let the authors know.
-
-	And if we keep it, we should probably add some sort of rate limiting to it.
-	"""
-	name = "authcheck"
-	checkedRenderer = False
-	openRenderer = True
-
-	def _addChallenge(self, request):
-		"""adds a www-authenticate header to request.
-		"""
-		# We may have to figure out a way to admit different realms here,
-		# but for now just use our global realm
-		realm = base.getConfig("web", "realm")
-		request.setHeader('WWW-Authenticate', 
-			'Basic realm="%s"'%realm)
-
-	def render(self, request):
-		if request.getUser():
-			return self._renderWithUser(request)
-
-		else:
-			return self._renderWithoutUser(request)
-
-	def _renderWithUser(self, request):
-		"""creates a response when a request contains some credentials.
-
-		Four cases:
-
-		* bad credentials: return a 401 with an auth challenge.
-		* open service, good credentials: return 200 with the authenticated id
-		* closed service, good credentials: return 200 with the authenticated id 
-		* closed service, good credentials but not authorised: 
-		  return 401 with a challenge.
-		"""
-		groups = creds.getGroupsForUser(request.getUser(), request.getPassword())
-
-		if self.service.limitTo is None:
-			isAuth = bool(groups)
-		else:
-			isAuth = self.service.limitTo in groups
-
-		request.setHeader("content-type", "text/plain")
-		if isAuth:
-			request.setHeader("x-authenticated-id", request.getUser())
-			return b"PROBABLY YES"
-
-		else:
-			self._addChallenge(request)
-			request.setResponseCode(401)
-			return b"NO"
-	
-	def _renderWithoutUser(self, request):
-		"""creates a response when a request does not contain credentials.
-
-		Two cases:
-
-		* open service: return a 200 but annouce optional auth for the 
-		  sake of services that may support gratuitous auth.
-		* closed service: return 401 and a challenge.
-		"""
-		self._addChallenge(request)
-		request.setHeader("content-type", "text/plain")
-
-		if self.service.limitTo is None:
-			return b"PROBABLY YES"
-
-		request.setResponseCode(401)
-		return b"NO"
-
