@@ -2,7 +2,7 @@
 A renderer for Data to HTML/stan
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -16,7 +16,6 @@ import urllib.parse
 from twisted.web import template
 from twisted.web.template import tags as T
 
-from gavo import base
 from gavo import formal
 from gavo import formats
 from gavo import rsc
@@ -110,10 +109,10 @@ def _bibcodeMapperFactory(colDesc):
 	def coder(data):
 		if data:
 			for item in data.split(","):
-				yield T.a(href=base.getConfig("web", "adsMirror")+
-					"/abs/"+urllib.parse.quote(item.strip()))[
-					item.strip()]
-				yield ", "
+				urlBibcode = urllib.parse.quote(item.strip())
+				adsLink = f"https://ui.adsabs.harvard.edu/abs/{urlBibcode}/abstract"
+				yield T.a(href=adsLink)[item.strip()]
+				yield " "
 		else:
 			yield ""
 	return coder
