@@ -6,7 +6,7 @@ it this way (in effect, since they are supposed to be three different
 capabilities).
 """
 
-#c Copyright 2008-2021, the GAVO project <gavo@ari.uni-heidelberg.de>
+#c Copyright 2008-2022, the GAVO project <gavo@ari.uni-heidelberg.de>
 #c
 #c This program is free software, covered by the GNU GPL.  See the
 #c COPYING file in the source distribution.
@@ -24,6 +24,7 @@ from gavo.base import meta
 from gavo.registry import capabilities
 from gavo.registry import tableset
 from gavo.registry.model import VS
+from gavo.protocols import creds
 from gavo.utils.stanxml import Element, registerPrefix, schemaURL, xsiPrefix
 from gavo.web import grend
 from gavo.web import weberrors
@@ -148,6 +149,10 @@ class VOSICapabilityRenderer(VOSIRenderer):
 	An endpoint with this renderer is automatically registered for
 	every service.	The responses contain information on what renderers
 	("interfaces") are available for a service and what properties they have.
+
+	This also doubles as a canary for authentication, which is why there
+	are the somewhat complicated things in render; cf. 
+	https://wiki.ivoa.net/twiki/bin/view/IVOA/SSO_next
 	"""
 	name = "capabilities"
 
@@ -157,12 +162,73 @@ class VOSICapabilityRenderer(VOSIRenderer):
 		for pub in self.service.getPublicationsForSet(None):
 			try:
 				root = root[capabilities.getCapabilityElement(pub)]
-			except Exception:
+			except Exception as msg:
 				base.ui.notifyError("Error while creating VOSI capability"
-					" for %s"%(self.service.getURL(pub.render, absolute=False)))
+					" for %s: %s"%(
+					self.service.getURL(pub.render, absolute=False),
+					msg))
 
 		return root
 
+	def render(self, request):
+		if request.getUser():
+			return self._renderWithUser(request)
+
+		else:
+			return self._renderWithoutUser(request)
+
+	def _addChallenge(self, request):
+		"""adds a www-authenticate header to request.
+		"""
+		# We may have to figure out a way to admit different realms here,
+		# but for now just use our global realm
+		realm = base.getConfig("web", "realm")
+		request.setHeader('WWW-Authenticate', 
+			'Basic realm="%s"'%realm)
+
+	def _renderWithUser(self, request):
+		"""creates a response when a request contains some credentials.
+
+		Four cases:
+
+		* bad credentials: return a 401 with an auth challenge.
+		* open service, good credentials: return 200 with the authenticated id
+		* closed service, good credentials: return 200 with the authenticated id 
+		* closed service, good credentials but not authorised: 
+		  return 401 with a challenge.
+		"""
+		groups = creds.getGroupsForUser(request.getUser(), request.getPassword())
+
+		if self.service.limitTo is None:
+			isAuth = bool(groups)
+		else:
+			isAuth = self.service.limitTo in groups
+
+		request.setHeader("content-type", "text/plain")
+		if isAuth:
+			request.setHeader("x-vo-authenticated", request.getUser())
+
+		else:
+			self._addChallenge(request)
+			request.setResponseCode(401)
+
+		return super().render(request)
+	
+	def _renderWithoutUser(self, request):
+		"""creates a response when a request does not contain credentials.
+
+		Two cases:
+
+		* open service: return a 200 but annouce optional auth for the 
+		  sake of services that may support gratuitous auth.
+		* closed service: return 401 and a challenge.
+		"""
+		self._addChallenge(request)
+		if self.service.limitTo is not None:
+			request.setResponseCode(401)
+		return super().render(request)
+
+	
 
 class VOSITableResponse(VOSIRenderMixin, resource.Resource):
 	"""A resource building a VOSI tableset for a single table.
